import logging
import os

from plone import api
from plone.app.textfield.value import RichTextValue
from csv import reader
from six.moves import cStringIO


def create_toplevel_organizers_group(context):

    ps = api.portal.get_tool('portal_setup')
    ps.runImportStepFromProfile(
      'profile-sandia.conferencepolicy:default', 'typeinfo')
    ps.runImportStepFromProfile(
      'profile-sandia.conferencepolicy:default', 'rolemap')

    # create top level conference-orgranizers group
    api.group.create(
      groupname='site-conference-organizers',
      title='Conference Organizers',
      roles=['Conference Organizer', 'Reader'],
      )


def import_templates(context):
    portal = api.portal.get()
    template_folder = 'templates'
    if template_folder in portal:
        parent_folder = portal[template_folder]
    else:
        parent_folder = api.content.create(
            type='Folder',
            id=template_folder,
            title=template_folder,
            container=portal,
        )

    import_content(context, 'conference-templates/about', parent_folder, 'about')


def import_content(context, source, target, sub_dir):
    pages_dir = os.path.join(
        os.path.dirname(__file__), source, )
    # list_child_folders = [f.path for f in os.scandir(pages_dir) if f.is_dir()]

    rows = get_objects_data(context, pages_dir)
    portal = api.portal.get()
    # create target folder
    if sub_dir in target:
        parent_folder = target[sub_dir]
    else:
        parent_folder = api.content.create(
            type='Folder',
            id=sub_dir,
            title=sub_dir,
            container=target,
        )

    for id, title, type in rows:
        if type == "Folder":
            if id in parent_folder:
                sub_dir = parent_folder[id]
            else:
                sub_dir = api.content.create(
                    type=type,
                    title=title,
                    id=id,
                    container=parent_folder,
                )
        else:
            try:
                html_text = _load_html(id, pages_dir)
            except FileNotFoundError:
                continue

            if id in parent_folder:
                continue
            new_content = api.content.create(
                type=type,
                title=title,
                id=id,
                container=parent_folder,
                text=html_text,
            )

    set_default_page(parent_folder)


def set_default_page(parent_folder):
    if 'default-page' in parent_folder:
        parent_folder.setDefaultPage('default-page')


def get_objects_data(context, pages_dir):
    """Read file listing content to import"""
    objects = _read_file('.objects', pages_dir)
    if objects is None:
        logging.getLogger(__name__).info(
            "Could not find .objects file in %s" % pages_dir)
        return

    dialect = 'excel'
    stream = cStringIO(objects)

    rowiter = reader(stream, dialect)
    rows = [i for i in rowiter if i]
    return rows


def _read_file(filename, path):
    file_contents = None
    file_path = os.path.join(path, filename, )
    if file_path is not None:
        try:
            with open(file_path, 'r') as file_opened:
                file_contents = file_opened.read()
        except FileNotFoundError:
            pass
    return file_contents


def _load_html(file_id, file_dir):
    html_file = os.path.join(
                    file_dir,
                    '{0}.html'.format(file_id),)
    with open(html_file, 'r') as html_opened:
        file_text = html_opened.read()
    return RichTextValue(
        raw=file_text, mimeType='text/html',
        outputMimeType='text/html', encoding='utf-8',
    )
